@@ -3,10 +3,13 @@ use std::str::Utf8Error;
 use std::sync::PoisonError;
 
 use bigdecimal::BigDecimal;
+use bigdecimal::num_bigint::BigInt;
 use bytebuffer::ByteBuffer;
 use chrono::{DateTime, Utc};
 use quick_error::quick_error;
 
+use crate::chrono::TimeZone;
+use crate::Column;
 use crate::response::VoltResponseInfo;
 
 #[allow(dead_code)]
@@ -98,6 +101,7 @@ pub trait Value: Debug {
     fn marshal(&self, bytebuffer: &mut ByteBuffer);
     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8);
     fn to_value_string(&self) -> String;
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> where Self: Sized;
 }
 
 
@@ -133,6 +137,16 @@ impl Value for bool {
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_BIT_VALUE {
+            return Ok(Option::None);
+        }
+        if bs[0] == 0 {
+            return Ok(Some(false));
+        }
+        return Ok(Some(true));
+    }
 }
 
 
@@ -160,6 +174,19 @@ impl Value for BigDecimal {
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> where Self: Sized {
+        if bs == NULL_DECIMAL {
+            return Ok(Option::None);
+        }
+        let int = BigInt::from_signed_bytes_be(&*bs);
+        let decimal = BigDecimal::new(int, 12);
+        return Ok(Some(decimal));
+    }
+
+    // fn from_bytes(bs: Vec<u8>) -> Option<Self> {
+
+    // }
 }
 
 impl Value for i8 {
@@ -173,13 +200,22 @@ impl Value for i8 {
     }
 
     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
-     //   bytebuffer.write_i8(0);
+        //   bytebuffer.write_i8(0);
         bytebuffer.write_i8(*self);
     }
 
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_BIT_VALUE {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let value = buffer.read_i8()?;
+        return Ok(Some(value));
+    }
 }
 
 impl Value for u8 {
@@ -193,13 +229,22 @@ impl Value for u8 {
     }
 
     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
-      //  bytebuffer.write_u8(0);
+        //  bytebuffer.write_u8(0);
         bytebuffer.write_u8(*self);
     }
 
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_BIT_VALUE {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let value = buffer.read_u8()?;
+        return Ok(Some(value));
+    }
 }
 
 impl Value for i16 {
@@ -213,13 +258,22 @@ impl Value for i16 {
     }
 
     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
-    //    bytebuffer.write_i16(0);
+        //    bytebuffer.write_i16(0);
         bytebuffer.write_i16(*self);
     }
 
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_SHORT_VALUE {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let value = buffer.read_i16()?;
+        return Ok(Some(value));
+    }
 }
 
 impl Value for u16 {
@@ -233,13 +287,22 @@ impl Value for u16 {
     }
 
     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
-      //  bytebuffer.write_i16(0);
+        //  bytebuffer.write_i16(0);
         bytebuffer.write_u16(*self);
     }
 
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_SHORT_VALUE {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let value = buffer.read_u16()?;
+        return Ok(Some(value));
+    }
 }
 
 impl Value for i32 {
@@ -253,13 +316,22 @@ impl Value for i32 {
     }
 
     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
-     //   bytebuffer.write_i32(0);
+        //   bytebuffer.write_i32(0);
         bytebuffer.write_i32(*self);
     }
 
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_INT_VALUE {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let value = buffer.read_i32()?;
+        return Ok(Some(value));
+    }
 }
 
 impl Value for u32 {
@@ -273,13 +345,22 @@ impl Value for u32 {
     }
 
     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
-     //   bytebuffer.write_u32(0);
+        //   bytebuffer.write_u32(0);
         bytebuffer.write_u32(*self);
     }
 
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_INT_VALUE {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let value = buffer.read_u32()?;
+        return Ok(Some(value));
+    }
 }
 
 impl Value for i64 {
@@ -293,13 +374,22 @@ impl Value for i64 {
     }
 
     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
-     //   bytebuffer.write_i64(0);
+        //   bytebuffer.write_i64(0);
         bytebuffer.write_i64(*self);
     }
 
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_LONG_VALUE {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let value = buffer.read_i64()?;
+        return Ok(Some(value));
+    }
 }
 
 impl Value for u64 {
@@ -313,13 +403,22 @@ impl Value for u64 {
     }
 
     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
-       // bytebuffer.write_u64(0);
+        // bytebuffer.write_u64(0);
         bytebuffer.write_u64(*self);
     }
 
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_LONG_VALUE {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let value = buffer.read_u64()?;
+        return Ok(Some(value));
+    }
 }
 
 impl Value for f64 {
@@ -339,6 +438,15 @@ impl Value for f64 {
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_FLOAT_VALUE {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let value = buffer.read_f64()?;
+        return Ok(Some(value));
+    }
 }
 
 
@@ -359,6 +467,29 @@ impl Value for String {
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, table_column: &Column) -> Result<Option<Self>, VoltError> {
+        return match table_column.header_type {
+            STRING_COLUMN => {
+                if bs == NULL_VARCHAR {
+                    return Ok(Option::None);
+                }
+                let mut buffer = ByteBuffer::from_bytes(&bs);
+                Ok(Option::Some(buffer.read_string()?))
+            }
+            _ => {
+                let res = crate::table::VoltTable::get_value_by_idx_column(table_column, bs)?;
+                match res {
+                    Some(v) => {
+                        Ok(Option::Some(v.to_value_string()))
+                    }
+                    None => {
+                        Ok(Option::None)
+                    }
+                }
+            }
+        };
+    }
 }
 
 impl Value for &str {
@@ -379,8 +510,11 @@ impl Value for &str {
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
-}
 
+    fn from_bytes(_bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        todo!()
+    }
+}
 
 impl Value for Vec<u8> {
     fn get_write_length(&self) -> i32 {
@@ -401,29 +535,51 @@ impl Value for Vec<u8> {
     fn to_value_string(&self) -> String {
         return format!("{:?}", self);
     }
-}
 
-impl Value for [u8] {
-    fn get_write_length(&self) -> i32 {
-        return (5 + self.len()) as i32;
-    }
-
-    fn marshal(&self, bytebuffer: &mut ByteBuffer) {
-        bytebuffer.write_i8(VAR_BIN_COLUMN);
-        bytebuffer.write_u32(self.len() as u32);
-        bytebuffer.write_bytes(&self);
-    }
-
-    fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
-        bytebuffer.write_u32(self.len() as u32);
-        bytebuffer.write_bytes(&self);
-    }
-
-    fn to_value_string(&self) -> String {
-        return format!("{:?}", self);
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> {
+        if bs == NULL_VARCHAR {
+            return Ok(Option::None);
+        }
+        let mut cp = bs.clone();
+        cp.drain(0..4);
+        return Ok(Option::Some(cp));
     }
 }
 
+// impl Value for [u8] {
+//     fn get_write_length(&self) -> i32 {
+//         return (5 + self.len()) as i32;
+//     }
+//
+//     fn marshal(&self, bytebuffer: &mut ByteBuffer) {
+//         bytebuffer.write_i8(VAR_BIN_COLUMN);
+//         bytebuffer.write_u32(self.len() as u32);
+//         bytebuffer.write_bytes(&self);
+//     }
+//
+//     fn marshal_in_table(&self, bytebuffer: &mut ByteBuffer, _column_type: i8) {
+//         bytebuffer.write_u32(self.len() as u32);
+//         bytebuffer.write_bytes(&self);
+//     }
+//
+//     fn to_value_string(&self) -> String {
+//         return format!("{:?}", self);
+//     }
+//
+//     fn from_bytes(bs: Vec<u8>) -> Result<Option<Self>, VoltError> where Self: Sized {
+//         todo!()
+//     }
+//
+//     // fn from_bytes(bs: Vec<u8>) -> Result<Option<Self>, VoltError> where Self: Sized {
+//     //     if bs == NULL_VARCHAR {
+//     //         return Ok(Option::None);
+//     //     }
+//     //     let mut res = bs.clone();
+//     //     res.drain(0..4);
+//     //     return Ok(Option::Some(*bs));
+//     // }
+// }
+
 impl Value for DateTime<Utc> {
     fn get_write_length(&self) -> i32 {
         return 9;
@@ -440,6 +596,15 @@ impl Value for DateTime<Utc> {
     fn to_value_string(&self) -> String {
         return self.to_string();
     }
+
+    fn from_bytes(bs: Vec<u8>, _column: &Column) -> Result<Option<Self>, VoltError> where Self: Sized {
+        if bs == NULL_TIMESTAMP {
+            return Ok(Option::None);
+        }
+        let mut buffer = ByteBuffer::from_bytes(&bs);
+        let time = buffer.read_i64()?;
+        return Ok(Option::Some(Utc.timestamp_millis(time / 1000)));
+    }
 }
 
 #[cfg(test)]
@@ -506,14 +671,14 @@ mod tests {
         println!("{:?} {} ", err, err);
     }
 
-    #[test]
-    fn test_vec() {
-        let xs: [u8; 5] = [1, 2, 3, 4, 5];
-        println!("{}", xs.get_write_length());
-        let mut ve = Vec::new();
-        ve.push(1 as u8);
-        ve.push(10 as u8);
-        println!("{}", &ve.get_write_length());
-    }
+    // #[test]
+    // fn test_vec() {
+    //     let xs: [u8; 5] = [1, 2, 3, 4, 5];
+    //     println!("{}", xs.get_write_length());
+    //     let mut ve = Vec::new();
+    //     ve.push(1 as u8);
+    //     ve.push(10 as u8);
+    //     println!("{}", &ve.get_write_length());
+    // }
 }
 
